@@ -9,8 +9,8 @@
 #include "field.h"
 #include "kpkpass_export.h"
 
+#include <QList>
 #include <QObject>
-#include <QVector>
 
 #include <memory>
 
@@ -52,7 +52,14 @@ class KPKPASS_EXPORT Pass : public QObject
     Q_PROPERTY(QColor labelColor READ labelColor CONSTANT)
     Q_PROPERTY(QString logoText READ logoText CONSTANT)
 
-    // needs to be QVariantList just for QML (Grantlee would also work with QVector<Field>
+    Q_PROPERTY(bool hasIcon READ hasIcon CONSTANT)
+    Q_PROPERTY(bool hasLogo READ hasLogo CONSTANT)
+    Q_PROPERTY(bool hasStrip READ hasStrip CONSTANT)
+    Q_PROPERTY(bool hasBackground READ hasBackground CONSTANT)
+    Q_PROPERTY(bool hasFooter READ hasFooter CONSTANT)
+    Q_PROPERTY(bool hasThumbnail READ hasThumbnail CONSTANT)
+
+    // needs to be QVariantList just for QML (Grantlee would also work with QList<Field>
     Q_PROPERTY(QVariantList barcodes READ barcodesVariant CONSTANT)
     Q_PROPERTY(QVariantList auxiliaryFields READ auxiliaryFieldsVariant CONSTANT)
     Q_PROPERTY(QVariantList backFields READ backFieldsVariant CONSTANT)
@@ -68,33 +75,45 @@ public:
     /** Type of the pass. */
     enum Type { BoardingPass, Coupon, EventTicket, Generic, StoreCard };
     Q_ENUM(Type)
-    Q_REQUIRED_RESULT Type type() const;
+    [[nodiscard]] Type type() const;
 
     // standard keys
-    Q_REQUIRED_RESULT QString description() const;
-    Q_REQUIRED_RESULT QString organizationName() const;
-    Q_REQUIRED_RESULT QString passTypeIdentifier() const;
-    Q_REQUIRED_RESULT QString serialNumber() const;
+    [[nodiscard]] QString description() const;
+    [[nodiscard]] QString organizationName() const;
+    [[nodiscard]] QString passTypeIdentifier() const;
+    [[nodiscard]] QString serialNumber() const;
 
     // expiration keys
-    Q_REQUIRED_RESULT QDateTime expirationDate() const;
-    Q_REQUIRED_RESULT bool isVoided() const;
+    [[nodiscard]] QDateTime expirationDate() const;
+    [[nodiscard]] bool isVoided() const;
 
     // relevance keys
     /** Locations associated with this pass. */
-    Q_REQUIRED_RESULT QVector<Location> locations() const;
+    [[nodiscard]] QList<Location> locations() const;
     /** Distance in meters to any of the pass locations before this pass becomes relevant. */
-    Q_REQUIRED_RESULT int maximumDistance() const;
-    Q_REQUIRED_RESULT QDateTime relevantDate() const;
+    [[nodiscard]] int maximumDistance() const;
+    [[nodiscard]] QDateTime relevantDate() const;
 
     // visual appearance keys
     /** Returns all barcodes defined in the pass. */
-    Q_REQUIRED_RESULT QVector<Barcode> barcodes() const;
-    Q_REQUIRED_RESULT QColor backgroundColor() const;
-    Q_REQUIRED_RESULT QColor foregroundColor() const;
-    Q_REQUIRED_RESULT QString groupingIdentifier() const;
-    Q_REQUIRED_RESULT QColor labelColor() const;
-    Q_REQUIRED_RESULT QString logoText() const;
+    [[nodiscard]] QList<Barcode> barcodes() const;
+    [[nodiscard]] QColor backgroundColor() const;
+    [[nodiscard]] QColor foregroundColor() const;
+    [[nodiscard]] QString groupingIdentifier() const;
+    [[nodiscard]] QColor labelColor() const;
+    [[nodiscard]] QString logoText() const;
+
+    /** Returns @c true if an image asset with the given base name exists.
+     *  @param baseName The name of the asset, without the file type and high dpi extensions.
+     *  @since 5.20.41
+     */
+    bool hasImage(const QString &baseName) const;
+    bool hasIcon() const;
+    bool hasLogo() const;
+    bool hasStrip() const;
+    bool hasBackground() const;
+    bool hasFooter() const;
+    bool hasThumbnail() const;
 
     /** Returns an image asset of this pass.
      *  @param baseName The name of the asset, without the file name extension.
@@ -115,29 +134,36 @@ public:
     QImage thumbnail(unsigned int devicePixelRatio = 1) const;
 
     // web service keys
-    Q_REQUIRED_RESULT QString authenticationToken() const;
-    Q_REQUIRED_RESULT QUrl webServiceUrl() const;
+    [[nodiscard]] QString authenticationToken() const;
+    [[nodiscard]] QUrl webServiceUrl() const;
     /** Pass update URL.
      * @see https://developer.apple.com/library/content/documentation/PassKit/Reference/PassKit_WebService/WebService.html
      */
-    Q_REQUIRED_RESULT QUrl passUpdateUrl() const;
+    [[nodiscard]] QUrl passUpdateUrl() const;
 
-    QVector<Field> auxiliaryFields() const;
-    QVector<Field> backFields() const;
-    QVector<Field> headerFields() const;
-    QVector<Field> primaryFields() const;
-    QVector<Field> secondaryFields() const;
+    QList<Field> auxiliaryFields() const;
+    QList<Field> backFields() const;
+    QList<Field> headerFields() const;
+    QList<Field> primaryFields() const;
+    QList<Field> secondaryFields() const;
 
     /** Returns the field with key @p key. */
     Field field(const QString &key) const;
     /** Returns all fields found in this pass. */
-    QVector<Field> fields() const;
+    QList<Field> fields() const;
 
     /** Create a appropriate sub-class based on the pkpass file type. */
     static Pass *fromData(const QByteArray &data, QObject *parent = nullptr);
     /** Create a appropriate sub-class based on the pkpass file type. */
     static Pass *fromFile(const QString &fileName, QObject *parent = nullptr);
 
+    /** The raw data of this pass.
+     *  That is the binary representation of the ZIP archive which contains
+     *  all the pass data.
+     *  @since 5.20.41
+     */
+    QByteArray rawData() const;
+
 protected:
     ///@cond internal
     friend class Barcode;
@@ -159,4 +185,3 @@ private:
 };
 
 }
-
